
RELEASE_VERSION='0.0.405166560'
BUILD_DATE='Tue Nov  9 12:18:50 UTC 2021'
BUILD_MACHINE_INFO='Linux runner-t4krtqay-project-28359933-concurrent-09pqkx 5.4.129-62.227.amzn2.x86_64 #1 SMP Wed Jul 7 00:08:43 UTC 2021 x86_64 GNU/Linux'
GITLAB_CI_JOB_ID='1763689788'
GIT_COMMIT_SHA='22e4b736294872e03326b14bbca45add91002d4c'
GIT_BRANCH='main'
